let serverURL;
//let isHTTPS;
let serverDeviceID;
let loginUploadURL;
const loginURL = "/patron/v23/auth";
let printJob;
let printer;

(function () {

	const initialise = function() {

		document.title = "GoPrint Driver";

		chrome.storage.local.set({"username": ""});
		chrome.storage.local.set({"passcode": ""});

        $("#login").click(function() {
			login();
		});

		$('#password').keydown(function(event) {
	        if (event.keyCode === 13) {
				login();
	        }
      	});

        $("#failDialog").on("shown.bs.modal", function() {
    		$("#loading").modal('hide');
        });

        $("#failDialogWithReason").on("shown.bs.modal", function() {
    		$("#loading").modal('hide');
        });

        $("#timeOutDialog").on("shown.bs.modal", function() {
    		$("#loading").modal('hide');
        });

        $("#bt-settings").click(function() {
        	openSettingsScreen();
		});

	    chrome.storage.local.get(['endPoint'], function(result) {
			if(result.endPoint === undefined) {
				serverURL = "https://portal.goprint.cloud";
				chrome.storage.local.set({"endPoint": serverURL});
			} else {
				serverURL = result.endPoint;
			}
		});

		chrome.storage.local.get(['deviceID'], function(result) {
			if(result.deviceID === undefined) {
				openSettingsScreen();
			} else {
				serverDeviceID = result.deviceID;
			}
		});

		chrome.storage.local.get(['printer'], function(result) {
			printer = result.printer;

			if(printer != undefined) {
				
				$("#username").attr('placeholder', printer.userNamePrompt);
				$("#password").attr('placeholder', printer.passwordPrompt);
				
				$("#mandatoryFieldsMsg").html(printer.userNamePrompt + " and " + printer.passwordPrompt + " are required");
			}
		});
		
        $("#username").focus();
	};

    const openSettingsScreen = function() {
    	let screenWidth = screen.availWidth;
	    let screenHeight = screen.availHeight;
		const pWidth = 320, pHeight = 300;

		chrome.windows.getCurrent(function(wind) {
			screenWidth = wind.width;
			screenHeight = wind.height;
		
			chrome.windows.create({
				url: "settings.html",
				type: "popup",
				width: pWidth,
				height: pHeight
			});
		});	

		chrome.runtime.onMessage.addListener(
	    	function(request, sender, sendResponse) {
	    		if(request == "refreshSettings") {
	    			openPreLoginScreen();
	    		}
			}
		);
    };


	const openPreLoginScreen = function() {
		let screenWidth = screen.availWidth;
	    let screenHeight = screen.availHeight;
		const pWidth = 529, pHeight = 536;

		chrome.windows.getCurrent(function(wind) {
			screenWidth = wind.width;
			screenHeight = wind.height;
		
			chrome.windows.create({
				url: "pre_login.html",
				type: "popup",
				width: pWidth,
				height: pHeight
			});

			window.close();
		});
	};
	
	const login = function() {

		const loginRequest = () => {
	        return new Promise((resolve, reject) => {


	        	if(serverURL != null 
	        		&& serverURL.length > 0 
	        		&& serverDeviceID != null
	        		&& serverDeviceID.length > 0) {

					let errorMsg = "";
	        		if($("#username").val() === "") {
        				errorMsg += "Username is required.";
	        		}
	        		if($("#password").val() === "") {
	        			if(errorMsg !== "") {
	        				errorMsg += "<br/>";
	        			}
        				errorMsg += "Password is required.";
	        		}

	        		if(errorMsg === "") {
						loginUploadURL = serverURL + loginURL;

			            let xhr = new XMLHttpRequest();
			            //xhr.withCredentials = true;

			            xhr.addEventListener("readystatechange", function() {
					  		if(this.readyState === 4) {
					    		console.log(this.responseText);
					  		}
						});

			            try {
			                xhr.open("GET", loginUploadURL);
			            } catch (e) {
			                console.log("Ignoring invalid url: " + loginUploadURL);
			            }

			            xhr.timeout = 45000;
			            xhr.setRequestHeader('v-device-id', serverDeviceID);
			            xhr.setRequestHeader('v-credential', $("#username").val());
			            xhr.setRequestHeader('v-user-pin', $("#password").val());

			            xhr.onload = function () {

			                if (xhr.status >= 200 && xhr.status < 300) {
			                    var success = onUploadFinished(xhr);

			                    resolve(success);

			                } else {
			                    console.log("request to " + loginUploadURL + " failed", this.status, xhr.statusText);
			                	resolve("request_failed");
			                }
			            };
			            xhr.ontimeout = function (e) {
			                console.log("timed out", this.status, xhr.statusText);
			                resolve("timed_out");
			            };
			            xhr.onerror = function (e) {
			                console.log("request failed", this.status, xhr.statusText);
			                resolve("request_failed");
			            };


			            xhr.send();
			        }
		        } else {
		        	resolve("");
		        }

	        });
	    }

		if($("#username").val() === "" || $("#password").val() === "") {
    		$("#mandatoryFieldsDialog").modal('show');
	    } else {
			$("#loading").modal('show');
		    loginRequest().then(values => {
		        
				$("#loading").modal('hide');
		        // Set the result popup fields
	            if(values.authorized) {
	            	chrome.storage.local.set({"username": $("#username").val()});
	            	chrome.storage.local.set({"passcode": $("#password").val()});
					openUploadScreen();
	            } else {
	            	if(values != undefined && values.note != '') {
	            		$("#failDialogWithReason").modal('show');
	            		$("#errorMessage").html(values.note);
	            	} else {
	            		$("#failDialog").modal('show');
	            	}
					
	            }
	        });
		}
	};

	const openUploadScreen = function() {
		let screenWidth = screen.availWidth;
		    let screenHeight = screen.availHeight;
			const pWidth = 529, pHeight = 536;

			chrome.windows.getCurrent(function(wind) {
				screenWidth = wind.width;
				screenHeight = wind.height;
			
				chrome.windows.create({
					url: "upload.html",
					type: "popup",
					width: pWidth,
					height: pHeight
				});

				window.close();
			});
		};

	const setFields = function() {
		chrome.windows.getCurrent(function(wind) {

			let db;
			const request = indexedDB.open("GoPrintDB", 1);

			request.onerror = function(event) {
			  	console.error("Failed to open IndexedDB database", event.target.error);
			};
			request.onsuccess = function(event) {
			 	 db = event.target.result;
			  	console.log("Successfully opened IndexedDB database");

			  	const transaction = db.transaction(["GoPrintStore"], "readonly");
				const objectStore = transaction.objectStore("GoPrintStore");
				const requestStore = objectStore.get(1);

				requestStore.onsuccess = function(event) {
				  	console.log("Successfully retrieved data from IndexedDB database:", requestStore.result.value);

					printJob = requestStore.result.value;

					$("#file-name").text(printJob.title);
					$("#upload").click(function() {
						$("#loading").modal('show');
						uploadFile();
					});
					$('#successDialog').on('hidden.bs.modal', function () {
						window.close();
					});
					$('#failDialog').on('hidden.bs.modal', function () {
						$("#loading").modal('hide');
					});

				};
				requestStore.onerror = function(event) {
				  console.error("Failed to retrieve data from IndexedDB database", event.target.error);
				};
			};
			request.onupgradeneeded = function(event) {
			  const db = event.target.result;
			  const objectStore = db.createObjectStore("GoPrintStore", { keyPath: "id" });
			  console.log("IndexedDB database upgraded");
			};


			

			/*chrome.storage.local.get(['printJob'], function(result) {
				printJob = result.printJob;

				$("#file-name").text(printJob.title);
				$("#upload").click(function() {
					$("#loading").modal('show');
					uploadFile();
				});
				$('#successDialog').on('hidden.bs.modal', function () {
					window.close();
				});
				$('#failDialog').on('hidden.bs.modal', function () {
					$("#loading").modal('hide');
				});
			});*/
        });
	};

	const onUploadFinished = xhr => {
	    console.log("JSON response from: " + xhr.responseURL + ": \n" + xhr.responseText);
	    try {

	        return JSON.parse(xhr.responseText);
            
	    } catch (e) {
	        console.warn("Invalid JSON response from:" + xhr.responseURL + "error: " + e);
	        return "invalid response";
	    }

	};


	initialise();
    setFields();

})();