let serverURL;
//let isHTTPS;
//let serverDeviceID = "";
let serverDeviceID = "";
let serverUploadURL;
const uploadURL = "/netzcore/cloudprinting/uploadfile";
let printJob;
let printer;

(function () {

	const initialise = function() {

		document.title = "GoPrint Driver";

		chrome.storage.local.set({"username": ""});
		chrome.storage.local.set({"passcode": ""});

        $("#bt-user").click(function() {
        	openUserScreen();
        });

        $("#bt-guest").click(function() {
        	openUploadScreen();
        });

        $("#failDialog").on("shown.bs.modal", function() {
    		$("#loading").modal('hide');
        });

        $("#failDialogWithReason").on("shown.bs.modal", function() {
    		$("#loading").modal('hide');
        });

        $("#timeOutDialog").on("shown.bs.modal", function() {
    		$("#loading").modal('hide');
        });

	    chrome.storage.local.get(['endPoint'], function(result) {
			if(result.endPoint === undefined) {
				serverURL = "https://portal.goprint.cloud";
				chrome.storage.local.set({"endPoint": serverURL});
			} else {
				serverURL = result.endPoint;
			}
		});

		chrome.storage.local.get(['deviceID'], function(result) {
			if(result.deviceID !== undefined) {
				serverDeviceID = result.deviceID;
				chrome.storage.local.set({"deviceID": serverDeviceID});
			} else {
				openSettingsScreen();
			}
		});

		$("#pre-login-buttons").hide();

        $("#bt-settings").click(function() {
			openSettingsScreen();
		});

		chrome.storage.local.get(['printer'], function(result) {
				printer = result.printer;

				if(printer === undefined && (serverDeviceID === undefined || serverDeviceID === "")) {

					if(serverDeviceID === undefined || serverDeviceID === "") {
						//openSettingsScreen();
					} 
				} else {
					$("#loading").modal('show');

					getDevice(serverURL, serverDeviceID);
				}
			}
		);
	};

	const openPreLoginScreen = function() {
		let screenWidth = screen.availWidth;
	    let screenHeight = screen.availHeight;
		const pWidth = 529, pHeight = 536;

		chrome.windows.getCurrent(function(wind) {
			screenWidth = wind.width;
			screenHeight = wind.height;
		
			chrome.windows.create({
				url: "pre_login.html",
				type: "popup",
				width: pWidth,
				height: pHeight
			});

			window.close();
		});
	};

	const addSettingsListener = function() {
		chrome.runtime.onMessage.addListener(
	    	function(request, sender, sendResponse) {
	    		if(request === "refreshSettings") {

					$("#loading").modal('hide');
					chrome.storage.local.get(['printer'], function(result) {
						printer = result.printer;

						if(printer != undefined) {
							
							if(printer.loginOption && printer.guestOption) {
								$("#pre-login-buttons").show();
								$("#bt-user").show();
								$("#bt-guest").show();
							} else {
								if(printer.loginOption) {
									openUserScreen();
								} else {
									openUploadScreen();
								} 
							}
							
						}
					});

	    		}
			}
		);
	}

	const getDevice = function(endpoint, deviceID) {
		const getDeviceRequest = () => {
	        return new Promise((resolve, reject) => {

	        	if(endpoint != undefined
	        		&& deviceID != undefined
	        		&& deviceID.length > 0) {

		            var formData = new FormData();
		            formData.append("deviceID", deviceID);

		            let xhr = new XMLHttpRequest();

		            xhr.addEventListener("readystatechange", function() {
				  		if(this.readyState === 4) {
				    		console.log(this.responseText);
				  		}
					});

		            try {
		                xhr.open("POST", endpoint + "/netzcore/location/service/printerByID");
		            } catch (e) {
		                console.log("Ignoring invalid url: " + endpoint);
		            }

		            xhr.timeout = 10000;
		            xhr.onload = function () {

		                if (xhr.status >= 200 && xhr.status < 300) {
		                    var success = onGetDeviceFinished(xhr);

		                    resolve(success);

		                } else {
		                    console.log("request to " + endpoint + " failed", this.status, xhr.statusText);
		                	resolve("request_failed");
		                	$("#endpointNotFound").modal('show');
		                }
		            };
		            xhr.ontimeout = function (e) {
		                console.log("timed out", this.status, xhr.statusText);
		                resolve("timed_out");
		                $("#endpointNotFound").modal('show');
		            };
		            xhr.onerror = function (e) {
		                console.log("request failed", this.status, xhr.statusText);
		                resolve("request_failed");
		                $("#endpointNotFound").modal('show');
		            };


		            xhr.send(formData);
		        } else {
		        	resolve("");
		        }

	        });
	    }

		const onGetDeviceFinished = xhr => {
		    console.log("JSON response from: " + xhr.responseURL + ": \n" + xhr.responseText);
		    try {

		        return JSON.parse(xhr.responseText);
	            
		    } catch (e) {
		        console.warn("Invalid JSON response from:" + xhr.responseURL + "error: " + e);
		        return "invalid response";
		    }

		};

	    getDeviceRequest().then(values => {
	        
	        setTimeout(()=>{
				$("#loading").modal('hide');

				if(values !== "request_failed") {

				    // Extract the list from the JSON object
				    var printer = values.printer;

			        // Set the result popup fields
		            if(printer != undefined) {
						chrome.storage.local.set({"printer": printer});
						
						if(printer.loginOption && printer.guestOption) {
							$("#pre-login-buttons").show();
							$("#bt-user").show();
							$("#bt-guest").show();
						} else {
							if(printer.loginOption) {
								openUserScreen();
							} else {
								openUploadScreen();
							} 
						}

		            } else {
		        		$("#deviceNotFound").modal('show');
		            }

		        } else {
		        	$("#deviceNotFound").modal('show');
		        }
			}, 500);

			
		});
	};

	const openSettingsScreen = function() {
    	let screenWidth = screen.availWidth;
	    let screenHeight = screen.availHeight;
		const pWidth = 320, pHeight = 300;

		chrome.windows.getCurrent(function(wind) {
			screenWidth = wind.width;
			screenHeight = wind.height;
		
			chrome.windows.create({
				url: "settings.html",
				type: "popup",
				width: pWidth,
				height: pHeight
			});
		});	

		addSettingsListener();
	};

	const openUploadScreen = function() {
		let screenWidth = screen.availWidth;
		    let screenHeight = screen.availHeight;
			const pWidth = 529, pHeight = 536;

			chrome.windows.getCurrent(function(wind) {
				screenWidth = wind.width;
				screenHeight = wind.height;
			
				chrome.windows.create({
					url: "upload.html",
					type: "popup",
					width: pWidth,
					height: pHeight
				});

				window.close();
			});
		};

	const openUserScreen = function() {
		let screenWidth = screen.availWidth;
	    let screenHeight = screen.availHeight;
		const pWidth = 529, pHeight = 536;

		chrome.windows.getCurrent(function(wind) {
			screenWidth = wind.width;
			screenHeight = wind.height;
		
			chrome.windows.create({
				url: "login.html",
				type: "popup",
				width: pWidth,
				height: pHeight
			});

			window.close();

		});
	};


	initialise();

})();